/**
 * Copyright (c) 2011, Regents of the University of California
 * All rights reserved.
 * <p/>
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * <p/>
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * <p/>
 * 2. Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * <p/>
 * 3. Neither the name of the University of California, Berkeley nor
 * the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 * <p/>
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package edu.berkeley.cs.wise.concolic;

import java.io.FileNotFoundException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * @author Koushik Sen <ksen@cs.berkeley.edu>
 * @author Jacob Burnim <jburnim@cs.berkeley.edu>
 */
public class Concolic {
    public static Input input = new InputImpl();

    public static void ResetBranchCounting() {
        ObserverForConcolic.ResetBranchCounting();
    }

    public static void fastConcolic(String[] args)
            throws IllegalAccessException, InvocationTargetException, FileNotFoundException {

        final int maxIterations =
                Integer.parseInt(System.getProperty("concolic.iterations", "1000000"));

        Method m = null;
        String tmp[] = new String[args.length - 1];
        try {
            Class c = Class.forName(args[0]);
            Class[] argTypes = {args.getClass(),};
            m = c.getMethod("main", argTypes);

            for (int i = 0; i < tmp.length; i++) {
                tmp[i] = args[i + 1];
            }

        } catch (Exception e) {
            System.err.println(e);
            System.exit(1);
        }
        Object passedArgv[] = {tmp};

        int j = -1;
        while (++j < maxIterations) {
            if (j % 10000 == 0) {
                System.out.println("******** Iteration " + j + " ********");
            }
            try {
                m.invoke(null, passedArgv);
            } catch (Exception e2) {
                e2.printStackTrace();
            }
            if (!ObserverForConcolic.SolveAndReset()) {
                break;
            }
        }
        System.out.println("Finished after " + (j + 1) + " iterations.");

        if (j == maxIterations) {
            // We terminated the search early, so notify the observer.
            ObserverForConcolic.Shutdown();
        }
    }
}
