/**
 * Copyright (c) 2011, Regents of the University of California
 * All rights reserved.
 * <p/>
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * <p/>
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * <p/>
 * 2. Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * <p/>
 * 3. Neither the name of the University of California, Berkeley nor
 * the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 * <p/>
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package edu.berkeley.cs.wise.concolic.path;

import java.io.BufferedOutputStream;
import java.io.BufferedInputStream;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.Serializable;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashSet;

import edu.berkeley.cs.wise.concolic.Globals;

/**
 * @author Jacob Burnim <jburnim@cs.berkeley.edu>
 */
public class Summary<G> implements Serializable {

    public static <G> ArrayList<Summary<G>> readSummaries() {
        try {
            FileInputStream fis = new FileInputStream(Globals.summariesFile);
            ObjectInputStream in =
                new ObjectInputStream(new BufferedInputStream(fis));

            return (ArrayList<Summary<G>>)in.readObject();

        } catch (IOException e) {
            return new ArrayList<Summary<G>>();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
            System.exit(1);
        }
        return null;
    }

    public static <G> void writeSummaries(ArrayList<Summary<G>> summaries) {
        try {
            FileOutputStream fos = new FileOutputStream(Globals.summariesFile);
            ObjectOutputStream out =
                new ObjectOutputStream(new BufferedOutputStream(fos));

            out.writeObject(summaries);
            out.close();
        } catch (IOException e) {
            System.err.println("Failed to write to: " + Globals.summariesFile);
            System.exit(1);
        }
    }

    private LinkedHashSet<G> maxGenerators;
    private HashMap<G, Integer> pathCounts;

    public Summary(LinkedHashSet<G> max, HashMap<G, Integer> count) {
        maxGenerators = max;
        pathCounts = count;
    }

    public LinkedHashSet<G> getMaxGenerators() {
        return maxGenerators;
    }

    public HashMap<G, Integer> getPathCounts() {
        return pathCounts;
    }
}
